@@ -15,7 +15,7 @@ type 'a t =
   ; next_size : int
   ; ranges : 'a Range.t list
   }
-[@@deriving fields, sexp_of]
+[@@deriving fields ~getters, sexp_of]
 
 (** [all_same t] returns true if [t] contains only Same ranges. *)
 val all_same : 'a t -> bool
@@ -24,7 +24,14 @@ val all_same : 'a t -> bool
 val concat_map : 'a t -> f:('a Range.t -> 'b Range.t list) -> 'b t
 
 module Stable : sig
-  module V1 : sig
+  module V2 : sig
     type nonrec 'a t = 'a t [@@deriving sexp, bin_io]
   end
+
+  module V1 : sig
+    type nonrec 'a t [@@deriving sexp, bin_io]
+
+    val to_v2 : 'a t -> 'a V2.t
+    val of_v2_no_moves_exn : 'a V2.t -> 'a t
+  end
 end
