@@ -4,6 +4,7 @@ module Hunk = Hunk
 module Hunks = Hunks
 module Matching_block = Matching_block
 module Range = Range
+module Move_id = Move_id
 
 let ( <|> ) ar (i, j) = if j <= i then [||] else Array.slice ar i j
 
@@ -275,23 +276,23 @@ module Make (Elt : Hashtbl.Key) = struct
       let x = bravo.(x's_pos_in_b) in
       Hashtbl.find unique x
       |> Option.iter ~f:(fun pos ->
-        match pos with
-        | Not_unique { occurrences_in_a = n } ->
-          if n > 0
-          then (
-            Hashtbl.set unique ~key:x ~data:(Not_unique { occurrences_in_a = n - 1 });
-            incr intersection_size)
-        | Unique_in_a { index_in_a = x's_pos_in_a } ->
-          incr num_pairs;
-          incr intersection_size;
-          Hashtbl.set
-            unique
-            ~key:x
-            ~data:
-              (Unique_in_a_b { index_in_a = x's_pos_in_a; index_in_b = x's_pos_in_b })
-        | Unique_in_a_b _ ->
-          decr num_pairs;
-          Hashtbl.set unique ~key:x ~data:(Not_unique { occurrences_in_a = 0 }))
+           match pos with
+           | Not_unique { occurrences_in_a = n } ->
+             if n > 0
+             then (
+               Hashtbl.set unique ~key:x ~data:(Not_unique { occurrences_in_a = n - 1 });
+               incr intersection_size)
+           | Unique_in_a { index_in_a = x's_pos_in_a } ->
+             incr num_pairs;
+             incr intersection_size;
+             Hashtbl.set
+               unique
+               ~key:x
+               ~data:
+                 (Unique_in_a_b { index_in_a = x's_pos_in_a; index_in_b = x's_pos_in_b })
+           | Unique_in_a_b _ ->
+             decr num_pairs;
+             Hashtbl.set unique ~key:x ~data:(Not_unique { occurrences_in_a = 0 }))
     done;
     (* If we're ignoring almost all of the text when we perform the patience
        diff algorithm, it will often give bad results. *)
@@ -373,11 +374,11 @@ module Make (Elt : Hashtbl.Key) = struct
           | `Computed_lcs lcs ->
             lcs
             |> List.iter ~f:(fun (apos, bpos) ->
-              if !last_a_pos + 1 <> apos || !last_b_pos + 1 <> bpos
-              then recurse_matches (!last_a_pos + 1) (!last_b_pos + 1) apos bpos;
-              last_a_pos := apos;
-              last_b_pos := bpos;
-              add_match (apos, bpos));
+                 if !last_a_pos + 1 <> apos || !last_b_pos + 1 <> bpos
+                 then recurse_matches (!last_a_pos + 1) (!last_b_pos + 1) apos bpos;
+                 last_a_pos := apos;
+                 last_b_pos := bpos;
+                 add_match (apos, bpos));
             if !matches_ref_length > old_length (* Did unique_lcs find anything at all? *)
             then recurse_matches (!last_a_pos + 1) (!last_b_pos + 1) ahi bhi
             else plain_diff ())
@@ -437,8 +438,8 @@ module Make (Elt : Hashtbl.Key) = struct
   ;;
 
   let change_between
-        (left_matching_block : Matching_block.t)
-        (right_matching_block : Matching_block.t)
+    (left_matching_block : Matching_block.t)
+    (right_matching_block : Matching_block.t)
     =
     max
       (right_matching_block.prev_start - left_matching_block.prev_start)
@@ -459,20 +460,20 @@ module Make (Elt : Hashtbl.Key) = struct
             other_blocks
             ~init:([], first_block)
             ~f:(fun (ans, pending) current_block ->
-              let rec loop ans pending =
-                match ans with
-                | [] -> ans, pending
-                | hd :: tl ->
-                  if should_discard_match
-                       ~big_enough
-                       ~left_change:(change_between hd pending)
-                       ~right_change:(change_between pending current_block)
-                       ~block_len:pending.length
-                  then loop tl hd
-                  else ans, pending
-              in
-              let updated_ans, updated_pending = loop ans pending in
-              updated_pending :: updated_ans, current_block)
+            let rec loop ans pending =
+              match ans with
+              | [] -> ans, pending
+              | hd :: tl ->
+                if should_discard_match
+                     ~big_enough
+                     ~left_change:(change_between hd pending)
+                     ~right_change:(change_between pending current_block)
+                     ~block_len:pending.length
+                then loop tl hd
+                else ans, pending
+            in
+            let updated_ans, updated_pending = loop ans pending in
+            updated_pending :: updated_ans, current_block)
         in
         List.rev (final_pending :: final_ans))
   ;;
@@ -495,26 +496,26 @@ module Make (Elt : Hashtbl.Key) = struct
             other_blocks
             ~init:([], first_block, second_block)
             ~f:(fun (ans, pendingA, pendingB) current_block ->
-              let rec loop ans pendingA pendingB =
-                match ans with
-                | [] -> ans, pendingA, pendingB
-                | hd :: tl ->
-                  if should_discard_match
-                       ~big_enough
-                       ~left_change:(change_between hd pendingA)
-                       ~right_change:(change_between pendingB current_block)
-                       ~block_len:
-                         (pendingB.length
-                          + min
-                              (pendingB.prev_start - pendingA.prev_start)
-                              (pendingB.next_start - pendingA.next_start))
-                  then loop tl hd pendingA
-                  else ans, pendingA, pendingB
-              in
-              let updated_ans, updated_pendingA, updated_pendingB =
-                loop ans pendingA pendingB
-              in
-              updated_pendingA :: updated_ans, updated_pendingB, current_block)
+            let rec loop ans pendingA pendingB =
+              match ans with
+              | [] -> ans, pendingA, pendingB
+              | hd :: tl ->
+                if should_discard_match
+                     ~big_enough
+                     ~left_change:(change_between hd pendingA)
+                     ~right_change:(change_between pendingB current_block)
+                     ~block_len:
+                       (pendingB.length
+                        + min
+                            (pendingB.prev_start - pendingA.prev_start)
+                            (pendingB.next_start - pendingA.next_start))
+                then loop tl hd pendingA
+                else ans, pendingA, pendingB
+            in
+            let updated_ans, updated_pendingA, updated_pendingB =
+              loop ans pendingA pendingB
+            in
+            updated_pendingA :: updated_ans, updated_pendingB, current_block)
         in
         List.rev (final_pendingB :: final_pendingA :: final_ans)
         (* The loop above only deleted the second element of each pair we're supposed to
@@ -603,13 +604,13 @@ module Make (Elt : Hashtbl.Key) = struct
      of prev/next which are the same (when index ranges are missing, that missing
      chunk is unique to the array they are missing from). *)
   let align_diffs
-        ~prev_elts
-        ~prev_scorable
-        ~next_elts
-        ~next_scorable
-        ~max_slide
-        ~score
-        blocks
+    ~prev_elts
+    ~prev_scorable
+    ~next_elts
+    ~next_scorable
+    ~max_slide
+    ~score
+    blocks
     =
     if max_slide = 0
     then blocks
@@ -651,20 +652,20 @@ module Make (Elt : Hashtbl.Key) = struct
           let best_score = ref 0 in
           let offset_of_best_score = ref 0 in
           let score_initial
-                (left_block : Matching_block.t)
-                (right_block : Matching_block.t)
+            (left_block : Matching_block.t)
+            (right_block : Matching_block.t)
             =
             best_score
-            := score
-                 ~prev_left:(left_block.prev_start + left_block.length - 1)
-                 ~prev_right:right_block.prev_start
-                 ~next_left:(left_block.next_start + left_block.length - 1)
-                 ~next_right:right_block.next_start
+              := score
+                   ~prev_left:(left_block.prev_start + left_block.length - 1)
+                   ~prev_right:right_block.prev_start
+                   ~next_left:(left_block.next_start + left_block.length - 1)
+                   ~next_right:right_block.next_start
           in
           let rec try_to_slide_left
-                    ~i
-                    (left_block : Matching_block.t)
-                    (right_block : Matching_block.t)
+            ~i
+            (left_block : Matching_block.t)
+            (right_block : Matching_block.t)
             =
             let offset_into_left = left_block.length - i in
             if offset_into_left < 0
@@ -699,9 +700,9 @@ module Make (Elt : Hashtbl.Key) = struct
                 try_to_slide_left ~i:(i + 1) left_block right_block))
           in
           let rec try_to_slide_right
-                    ~i
-                    (left_block : Matching_block.t)
-                    (right_block : Matching_block.t)
+            ~i
+            (left_block : Matching_block.t)
+            (right_block : Matching_block.t)
             =
             let offset_into_left = left_block.length + i - 1 in
             if i > right_block.length
@@ -765,13 +766,13 @@ module Make (Elt : Hashtbl.Key) = struct
   ;;
 
   let get_matching_blocks
-        ~transform
-        ?(big_enough = 1)
-        ?(max_slide = 0)
-        ?(score = fun _ _ _ -> 100)
-        ~prev:prev_scorable
-        ~next:next_scorable
-        ()
+    ~transform
+    ?(big_enough = 1)
+    ?(max_slide = 0)
+    ?(score = fun _ _ _ -> 100)
+    ~prev:prev_scorable
+    ~next:next_scorable
+    ()
     =
     let prev = Array.map prev_scorable ~f:transform in
     let next = Array.map next_scorable ~f:transform in
@@ -811,15 +812,15 @@ module Make (Elt : Hashtbl.Key) = struct
             then (
               let prev_range = prev <|> (i, prev_index) in
               let next_range = next <|> (j, next_index) in
-              Some (Replace (prev_range, next_range)))
+              Some (Replace (prev_range, next_range, None)))
             else if i < prev_index
             then (
               let prev_range = prev <|> (i, prev_index) in
-              Some (Prev prev_range))
+              Some (Prev (prev_range, None)))
             else if j < next_index
             then (
               let next_range = next <|> (j, next_index) in
-              Some (Next next_range))
+              Some (Next (next_range, None)))
             else None
           in
           let l =
@@ -922,13 +923,16 @@ module Make (Elt : Hashtbl.Key) = struct
             | Unified _ ->
               (* get_ranges_rev never returns a Unified range *)
               assert false
-            | Next range ->
+            | Prev (_, Some _) | Next (_, Some _) | Replace (_, _, Some _) ->
+              (* get_ranges_rev never finds moves *)
+              assert false
+            | Next (range, None) ->
               let stop = bhi + Array.length range in
               ahi, stop
-            | Prev range ->
+            | Prev (range, None) ->
               let stop = ahi + Array.length range in
               stop, bhi
-            | Replace (a_range, b_range) ->
+            | Replace (a_range, b_range, None) ->
               let prev_stop = ahi + Array.length a_range in
               let next_stop = bhi + Array.length b_range in
               prev_stop, next_stop
@@ -965,20 +969,20 @@ module Make (Elt : Hashtbl.Key) = struct
       let length = ref 0 in
       List.iter matches ~f:(fun il ->
         if Array.for_all start ~f:Option.is_some
-        && List.mapi il ~f:(fun i x -> x = value_exn start.(i) + !length)
-           |> List.for_all ~f:(fun x -> x)
+           && List.mapi il ~f:(fun i x -> x = value_exn start.(i) + !length)
+              |> List.for_all ~f:(fun x -> x)
         then incr length
         else (
           if Array.for_all start ~f:Option.is_some
           then
             collapsed
-            := (Array.map start ~f:value_exn |> Array.to_list, !length) :: !collapsed;
+              := (Array.map start ~f:value_exn |> Array.to_list, !length) :: !collapsed;
           List.iteri il ~f:(fun i x -> start.(i) <- Some x);
           length := 1));
       if Array.for_all start ~f:Option.is_some && !length <> 0
       then
         collapsed
-        := (Array.map start ~f:value_exn |> Array.to_list, !length) :: !collapsed;
+          := (Array.map start ~f:value_exn |> Array.to_list, !length) :: !collapsed;
       List.rev !collapsed)
   ;;
 
@@ -1009,9 +1013,9 @@ module Make (Elt : Hashtbl.Key) = struct
       let list =
         Hashtbl.to_alist hashtbl
         |> List.filter_map ~f:(fun (a, l) ->
-          if List.length l = len - 1
-          then Some (a :: (List.sort l ~compare:compare_int_pair |> List.map ~f:snd))
-          else None)
+             if List.length l = len - 1
+             then Some (a :: (List.sort l ~compare:compare_int_pair |> List.map ~f:snd))
+             else None)
         |> List.sort ~compare:(List.compare Int.compare)
       in
       let matching_blocks = collapse_multi_sequences list in
@@ -1022,10 +1026,19 @@ module Make (Elt : Hashtbl.Key) = struct
         if Array.compare Int.compare last_pos ar' <> 0
         then
           merged_array
-          := Different (array_mapi2 last_pos ar' ~f:(fun i n m -> ar.(i) <|> (n, m)))
-             :: !merged_array;
+            := Different (array_mapi2 last_pos ar' ~f:(fun i n m -> ar.(i) <|> (n, m)))
+               :: !merged_array;
         merged_array := Same (ar.(0) <|> (ar'.(0), ar'.(0) + len)) :: !merged_array;
         Array.iteri last_pos ~f:(fun i _ -> last_pos.(i) <- ar'.(i) + len));
+      let trailing_lines =
+        Array.existsi last_pos ~f:(fun i last_pos -> Array.length ar.(i) > last_pos)
+      in
+      if trailing_lines
+      then
+        merged_array
+          := Different
+               (Array.mapi last_pos ~f:(fun i n -> ar.(i) <|> (n, Array.length ar.(i))))
+             :: !merged_array;
       List.rev !merged_array)
   ;;
 end
@@ -1068,7 +1081,7 @@ let%test_module _ =
       let check a b =
         let matches = P.matches a b in
         if is_increasing (-1) (List.map matches ~f:fst)
-        && is_increasing (-1) (List.map matches ~f:snd)
+           && is_increasing (-1) (List.map matches ~f:snd)
         then ()
         else
           failwiths
